import { ParameterType, } from "../utils/options/declaration.js";
import { nicePath } from "../utils/paths.js";
export class Outputs {
    application;
    outputs = new Map();
    defaultOutput = () => {
        return {
            name: "html",
            path: this.application.options.getValue("out"),
        };
    };
    constructor(application) {
        this.application = application;
    }
    addOutput(name, output) {
        if (this.outputs.has(name)) {
            throw new Error(`Output type '${name}' has already been defined`);
        }
        this.outputs.set(name, output);
    }
    setDefaultOutput(retriever) {
        this.defaultOutput = retriever;
    }
    async writeOutputs(project) {
        const options = this.application.options;
        let outputs = [];
        const outputShortcuts = options
            .getDeclarations()
            .filter((decl) => decl.type === ParameterType.Path && decl.outputShortcut);
        for (const shortcut of outputShortcuts) {
            if (options.isSet(shortcut.name)) {
                outputs.push({
                    name: shortcut.outputShortcut,
                    path: options.getValue(shortcut.name),
                });
            }
        }
        if (outputs.length === 0) {
            outputs = options.getValue("outputs") || [];
        }
        if (!outputs.length) {
            outputs.push(this.defaultOutput.call(null));
        }
        for (const output of outputs) {
            await this.writeOutput(output, project);
        }
    }
    async writeOutput(output, project) {
        const options = this.application.options;
        const snap = options.snapshot();
        const writer = this.outputs.get(output.name);
        if (!writer) {
            this.application.logger.error(this.application.i18n.specified_output_0_has_not_been_defined(output.name));
            return;
        }
        if (!this.application.setOptions(output.options || {}, true)) {
            options.restore(snap);
            return;
        }
        const preErrors = this.application.logger.errorCount;
        const start = Date.now();
        await writer(output.path, project);
        if (this.application.logger.errorCount === preErrors) {
            this.application.logger.info(this.application.i18n.output_0_generated_at_1(output.name, nicePath(output.path)));
        }
        else {
            this.application.logger.error(this.application.i18n.output_0_could_not_be_generated(output.name));
        }
        this.application.logger.verbose(`${output.name} took ${Date.now() - start}ms`);
        options.restore(snap);
    }
}
