import { JSX } from "../../../../utils/index.js";
import { getHierarchyRoots } from "../../lib.js";
function fullHierarchy(context, root, seen = new Set()) {
    if (seen.has(root))
        return;
    seen.add(root);
    // Note: We don't use root.anchor for the anchor, because those are built on a per page basis.
    // And classes/interfaces get their own page, so all the anchors will be empty anyways.
    // Full name should be safe here, since this list only includes classes/interfaces.
    return (JSX.createElement("li", null,
        JSX.createElement("a", { id: root.getFullName(), class: "tsd-anchor" }),
        JSX.createElement("a", { href: context.urlTo(root) },
            context.icons[root.kind](),
            root.name),
        JSX.createElement("ul", null,
            root.implementedBy?.map((child) => {
                return child.reflection && fullHierarchy(context, child.reflection, seen);
            }),
            root.extendedBy?.map((child) => {
                return child.reflection && fullHierarchy(context, child.reflection, seen);
            }))));
}
export function hierarchyTemplate(context, props) {
    return (JSX.createElement(JSX.Fragment, null,
        JSX.createElement("h2", null, context.i18n.theme_class_hierarchy_title()),
        getHierarchyRoots(props.project).map((root) => (JSX.createElement("ul", { class: "tsd-full-hierarchy" }, fullHierarchy(context, root))))));
}
